package http

import (
	"fmt"
	"net/http"
	"time"

	"git.javil.eu/jacob1123/budgeteer/postgres"
	"github.com/gin-gonic/gin"
	"github.com/google/uuid"
)

func getUUID(c *gin.Context, name string) (uuid.UUID, error) {
	value, succ := c.GetPostForm(name)
	if !succ {
		return uuid.UUID{}, fmt.Errorf("not set")
	}

	id, err := uuid.Parse(value)
	if err != nil {
		return uuid.UUID{}, fmt.Errorf("not a valid uuid: %w", err)
	}

	return id, nil
}

func getNullUUIDFromParam(c *gin.Context, name string) (uuid.NullUUID, error) {
	value := c.Param(name)
	if value == "" {
		return uuid.NullUUID{}, nil
	}

	id, err := uuid.Parse(value)
	if err != nil {
		return uuid.NullUUID{}, fmt.Errorf("not a valid uuid: %w", err)
	}

	return uuid.NullUUID{
		UUID:  id,
		Valid: true,
	}, nil
}

func getNullUUIDFromForm(c *gin.Context, name string) (uuid.NullUUID, error) {
	value, succ := c.GetPostForm(name)
	if !succ || value == "" {
		return uuid.NullUUID{}, nil
	}

	id, err := uuid.Parse(value)
	if err != nil {
		return uuid.NullUUID{}, fmt.Errorf("not a valid uuid: %w", err)
	}

	return uuid.NullUUID{
		UUID:  id,
		Valid: true,
	}, nil
}

func (h *Handler) newTransaction(c *gin.Context) {
	transactionMemo, _ := c.GetPostForm("memo")
	transactionAccountID, err := getUUID(c, "account_id")
	if err != nil {
		c.AbortWithError(http.StatusNotAcceptable, fmt.Errorf("account_id: %w", err))
		return
	}

	transactionCategoryID, err := getNullUUIDFromForm(c, "category_id")
	if err != nil {
		c.AbortWithError(http.StatusNotAcceptable, fmt.Errorf("category_id: %w", err))
		return
	}

	transactionDate, succ := c.GetPostForm("date")
	if !succ {
		c.AbortWithError(http.StatusNotAcceptable, fmt.Errorf("date missing"))
		return
	}

	transactionDateValue, err := time.Parse("2006-01-02", transactionDate)
	if err != nil {
		c.AbortWithError(http.StatusNotAcceptable, fmt.Errorf("date is not a valid date"))
		return
	}

	transactionAmount, succ := c.GetPostForm("amount")
	if !succ {
		c.AbortWithError(http.StatusNotAcceptable, fmt.Errorf("amount missing"))
		return
	}

	amount := postgres.Numeric{}
	amount.Set(transactionAmount)

	transactionUUID, err := getNullUUIDFromParam(c, "transactionid")
	if err != nil {

	}
	if !transactionUUID.Valid {
		new := postgres.CreateTransactionParams{
			Memo:       transactionMemo,
			Date:       transactionDateValue,
			Amount:     amount,
			AccountID:  transactionAccountID,
			PayeeID:    uuid.NullUUID{},
			CategoryID: transactionCategoryID,
		}
		_, err = h.Service.CreateTransaction(c.Request.Context(), new)
		if err != nil {
			c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("create transaction: %w", err))
			return
		}
	} else {
		_, delete := c.GetPostForm("delete")
		if delete {
			h.Service.DeleteTransaction(c.Request.Context(), transactionUUID.UUID)
		}
		update := postgres.UpdateTransactionParams{
			Memo:       transactionMemo,
			Date:       transactionDateValue,
			Amount:     amount,
			AccountID:  transactionAccountID,
			PayeeID:    uuid.NullUUID{},
			CategoryID: transactionCategoryID,
		}
		err = h.Service.UpdateTransaction(c.Request.Context(), update)
		if err != nil {
			c.AbortWithError(http.StatusInternalServerError, fmt.Errorf("update transaction: %w", err))
			return
		}
	}
}
